import { jsx } from 'react/jsx-runtime';
import { useRef, useState, useEffect, useCallback } from 'react';
import { select, pointer, selectAll } from 'd3-selection';
import { scaleLinear, scaleBand } from 'd3-scale';
import { axisBottom, axisLeft } from 'd3-axis';
import 'd3-transition';
import { easeCubic } from 'd3-ease';
import { line, lineRadial, curveLinearClosed, curveStepAfter } from 'd3-shape';
import { range } from 'd3-array';
import { drag } from 'd3-drag';

var defaultDimensions$3 = {
    chartHeight: 600,
    chartWidth: 800,
    marginLeft: 80,
    marginRight: 160,
    marginTop: 0,
    marginBottom: 0,
};
// Change me: add arg for reference point and use objective data just to set up.
var HorizontalBars = function (_a) {
    var objectiveData = _a.objectiveData, setReferencePoint = _a.setReferencePoint, referencePoint = _a.referencePoint, currentPoint = _a.currentPoint, dimensionsMaybe = _a.dimensionsMaybe;
    /*
    These hooks are used to store states, which are meaningful to the operation of this component.
    - ref should only ever point to the single svg-element
    - selection is used to have a usable hook to get hold of the DOM through d3 throughout this
      function
    - data is used to refer to the data being displayed
    - prefPointLocs refers to the locations of the pointers indicating preference on each horizontal bar
     */
    var ref = useRef(null);
    // SetStateAction<Selection<SVGSVGElement, unknown, HTMLElement, any> | null>
    var _b = useState(null), selection = _b[0], setSelection = _b[1];
    var data = useState(objectiveData.values[0].value.map(function (value, i) {
        return {
            name: objectiveData.names[i],
            value: value,
            direction: objectiveData.directions[i],
            selected: objectiveData.values[0].selected,
        };
    }))[0];
    var ideal = useState(objectiveData.ideal)[0];
    var nadir = useState(objectiveData.nadir)[0];
    var _c = useState(data.map(function (d) { return d.value; })), prefPointerLocs = _c[0], setPrefPointerLocs = _c[1];
    var _d = useState(data.map(function (d) { return d.value; })), infoPointerLocs = _d[0], setInfoPointerLocs = _d[1];
    var _e = useState(currentPoint), posNegMiddle = _e[0], setPosNegMiddle = _e[1];
    var dimensions = useState(dimensionsMaybe ? dimensionsMaybe : defaultDimensions$3)[0];
    useEffect(function () {
        setPrefPointerLocs(referencePoint);
        setInfoPointerLocs(referencePoint);
    }, [referencePoint]);
    useEffect(function () {
        setPosNegMiddle(currentPoint);
    }, [currentPoint]);
    // create an array of linear scales to scale each objective being maximized
    var xs = useCallback(function () {
        return data.map(function (_, i) {
            return scaleLinear()
                .domain([nadir[i], ideal[i]])
                .range([0, dimensions.chartWidth]);
        });
    }, [dimensions, data, ideal, nadir]);
    // create also an array of reverse scales to be used when minimizing
    var xs_rev = useCallback(function () {
        return data.map(function (_, i) {
            return scaleLinear()
                .domain([ideal[i], nadir[i]])
                .range([0, dimensions.chartWidth]);
        });
    }, [dimensions, data, ideal, nadir]);
    // create an array of bottom axises to work as the individual x-axis for each bar
    var xAxises = useCallback(function () {
        return data.map(function (d, i) {
            if (d.direction === -1) {
                return axisBottom(xs()[i]);
            }
            else {
                return axisBottom(xs_rev()[i]);
            }
        });
    }, [data, xs, xs_rev]);
    // create a discrete band to position each of the horizontal bars
    var y = useCallback(function () {
        return scaleBand()
            .domain(data.map(function (d) { return d.name; }))
            .range([0, dimensions.chartHeight])
            .padding(0.35);
    }, [dimensions, data]);
    // This is the main use effect and should really be fired only once per render.
    useEffect(function () {
        // create a discrete band to position each of the horizontal bars
        if (!selection) {
            // add svg and update selection
            var renderH = dimensions.chartHeight;
            var renderW = dimensions.chartWidth + dimensions.marginLeft + dimensions.marginRight;
            var newSelection = select(ref.current)
                .classed("svg-container", true)
                .append("svg")
                .attr("preserveAspectRatio", "xMinYMin meet")
                .attr("viewBox", "0 0 ".concat(renderW, " ").concat(renderH))
                .attr("viewBox", "0 0 ".concat(renderW, " ").concat(renderH))
                .classed("svg-content", true);
            // update selection
            setSelection(newSelection);
        }
        else {
            // clear the svg of its children
            console.log("svg clear!");
            selection.selectAll("*").remove();
            // Position the x-axises
            data.map(function (d, i) {
                return selection
                    .append("g")
                    .attr("transform", "translate(".concat(dimensions.marginLeft, ", ").concat(y().call(y, d.name) + y().bandwidth() * 1.05, ")"))
                    .call(xAxises()[i].tickSizeOuter(0));
            });
            // enter selection, append to this
            var enter = selection.append("g").selectAll("rect").data(data).enter();
            // append the labels for each bar positioned after the bar, roughly center to the bar
            enter
                .append("text")
                .attr("fill", "black")
                .attr("text-anchor", "start")
                .attr("transform", function (d) {
                return "translate(".concat(dimensions.marginLeft + dimensions.chartWidth + 10, ", ").concat(y()(d.name) + y().bandwidth() / 2, ")");
            })
                .text(function (d) { return "".concat(d.name, " ").concat(d.direction === -1 ? "(max)" : "(min)️"); })
                .attr("font-size", "14px")
                .attr("font-weight", "bold");
            // draw the positive space for max problems and negative space for min problems
            enter
                .append("rect")
                .attr("class", "posMaxNegMin")
                .attr("transform", "translate(".concat(dimensions.marginLeft, ", 0)"))
                .attr("width", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](d.value);
                }
                else {
                    return xs_rev()[i](d.value);
                }
            })
                .attr("y", function (d) {
                return y()(d.name);
            })
                .attr("x", 0)
                .attr("height", y().bandwidth)
                .attr("fill", function (d) {
                if (d.direction === -1) {
                    return "#90ee90";
                }
                else {
                    return "#ffcccb";
                }
            });
            // draw the positive space for min problems and the negative space for pos problems
            enter
                .append("rect")
                .attr("class", "negMaxPosMin")
                .attr("transform", "translate(".concat(dimensions.marginLeft, ", 0)"))
                .attr("width", function (d, i) {
                if (d.direction === -1) {
                    return dimensions.chartWidth - xs()[i](d.value);
                }
                else {
                    return dimensions.chartWidth - xs_rev()[i](d.value);
                }
            })
                .attr("y", function (d) {
                return y()(d.name);
            })
                .attr("x", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](d.value);
                }
                else {
                    return xs_rev()[i](d.value);
                }
            })
                .attr("height", y().bandwidth)
                .attr("fill", function (d) {
                if (d.direction === -1) {
                    return "#ffcccb";
                }
                else {
                    return "#90ee90";
                }
            });
            // draw the info pointers on each bar on top of the event overlay
            enter
                .append("line")
                .attr("class", "infoPointer")
                .attr("y1", function (d) { return y()(d.name); })
                .attr("y2", function (d) { return y()(d.name) + y().bandwidth(); })
                .attr("x1", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](d.value) + dimensions.marginLeft;
                }
                else {
                    return xs_rev()[i](d.value) + dimensions.marginLeft;
                }
            })
                .attr("x2", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](d.value) + dimensions.marginLeft;
                }
                else {
                    return xs_rev()[i](d.value) + dimensions.marginLeft;
                }
            })
                .attr("stroke-width", 2)
                .attr("stroke", "grey")
                .attr("fill", "none");
            // draw the preference pointers on each bar on top of the event overlay
            enter
                .append("line")
                .attr("class", "preferencePointer")
                .attr("y1", function (d) { return y()(d.name); })
                .attr("y2", function (d) { return y()(d.name) + y().bandwidth(); })
                .attr("x1", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](d.value) + dimensions.marginLeft;
                }
                else {
                    return xs_rev()[i](d.value) + dimensions.marginLeft;
                }
            })
                .attr("x2", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](d.value) + dimensions.marginLeft;
                }
                else {
                    return xs_rev()[i](d.value) + dimensions.marginLeft;
                }
            })
                .attr("stroke-width", 3)
                .attr("stroke", "black")
                .attr("fill", "none");
            enter
                .append("text")
                .attr("class", "infoLabel")
                .attr("fill", "black")
                .attr("text-anchor", function (d, i) {
                var midPoints = xs().map(function (x, _) {
                    return (x.domain()[0] + x.domain()[1]) / 2;
                });
                if (d.value >= midPoints[i]) {
                    return "end";
                }
                else {
                    return "start";
                }
            })
                .attr("transform", function (d, i) {
                return "translate(".concat(d.direction === -1
                    ? xs()[i](d.value) + dimensions.marginLeft
                    : xs_rev()[i](d.value) + dimensions.marginLeft, ", ").concat(y()(d.name) + y().bandwidth() / 2, ")");
            })
                .text(function (d) {
                return "".concat(d.name, ": ").concat(d.value, " ").concat(d.direction === -1 ? "(max)️" : "(min)️");
            })
                .attr("font-size", "16px")
                .attr("font-weight", "light");
            // draw a transparent overlay on top of each bar to work as an event detector
            enter
                .append("rect")
                .attr("class", "preferencePointerOverlay")
                .attr("transform", "translate(".concat(dimensions.marginLeft, ", 0)"))
                .attr("width", dimensions.chartWidth)
                .attr("y", function (d) {
                return y()(d.name);
            })
                .attr("x", 0)
                .attr("height", y().bandwidth)
                .attr("fill", "yellow")
                .attr("opacity", 0.0);
        }
    }, [selection, data, xs, xs_rev, y, xAxises, dimensions]);
    // update the negative and positive spaces when the current solution is updated
    useEffect(function () {
        if (!selection) {
            return;
        }
        var enterPosMaxNegMin = selection.selectAll(".posMaxNegMin").data(data);
        // update the positive space for max problems and negative space for min problems
        enterPosMaxNegMin.attr("width", function (d, i) {
            if (d.direction === -1) {
                return xs()[i](posNegMiddle[i]);
            }
            else {
                return xs_rev()[i](posNegMiddle[i]);
            }
        });
        var enterNegMaxPosMin = selection.selectAll(".negMaxPosMin").data(data);
        // draw the positive space for min problems and the negative space for pos problems
        enterNegMaxPosMin
            .attr("width", function (d, i) {
            if (d.direction === -1) {
                return dimensions.chartWidth - xs()[i](posNegMiddle[i]);
            }
            else {
                return dimensions.chartWidth - xs_rev()[i](posNegMiddle[i]);
            }
        })
            .attr("x", function (d, i) {
            if (d.direction === -1) {
                return xs()[i](posNegMiddle[i]);
            }
            else {
                return xs_rev()[i](posNegMiddle[i]);
            }
        });
    }, [data, posNegMiddle, dimensions]);
    /* We need an useEffect to watch for changes in the preference pointer values and update not
     *  just the location of the pointers, but also the event handlers on the overlay.
     */
    useEffect(function () {
        if (!selection) {
            return;
        }
        else {
            // select the preference pointers
            var enterPointer = selection.selectAll(".preferencePointer").data(data);
            // update the x location of the preference pointer according to the state of prefPointerLocs
            enterPointer
                .transition()
                .duration(100)
                .ease(easeCubic)
                .attr("x1", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](prefPointerLocs[i]) + dimensions.marginLeft;
                }
                else {
                    return xs_rev()[i](prefPointerLocs[i]) + dimensions.marginLeft;
                }
            })
                .attr("x2", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](prefPointerLocs[i]) + dimensions.marginLeft;
                }
                else {
                    return xs_rev()[i](prefPointerLocs[i]) + dimensions.marginLeft;
                }
            });
            // select the info pointers
            var enterInfoPtr = selection.selectAll(".infoPointer").data(data);
            // update the x location of the info pointer according to the state of infoPointerLocs
            enterInfoPtr
                .attr("x1", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](infoPointerLocs[i]) + dimensions.marginLeft;
                }
                else {
                    return xs_rev()[i](infoPointerLocs[i]) + dimensions.marginLeft;
                }
            })
                .attr("x2", function (d, i) {
                if (d.direction === -1) {
                    return xs()[i](infoPointerLocs[i]) + dimensions.marginLeft;
                }
                else {
                    return xs_rev()[i](infoPointerLocs[i]) + dimensions.marginLeft;
                }
            });
            // select the info labels
            var enterInfoLabels = selection.selectAll(".infoLabel").data(data);
            // update the x location of the info labels according to the state of infoPointerLocs
            enterInfoLabels
                .attr("transform", function (d, i) {
                return "translate(".concat(d.direction === -1
                    ? xs()[i](infoPointerLocs[i]) + dimensions.marginLeft
                    : xs_rev()[i](infoPointerLocs[i]) + dimensions.marginLeft, ", ").concat(y()(d.name) + y().bandwidth() / 2, ")");
            })
                .attr("text-anchor", function (_, i) {
                var midPoints = xs().map(function (x, _) {
                    return (x.domain()[0] + x.domain()[1]) / 2;
                });
                if (infoPointerLocs[i] >= midPoints[i]) {
                    return "end";
                }
                else {
                    return "start";
                }
            })
                .text(function (d, i) {
                return "".concat(d.name, ": ").concat(infoPointerLocs[i].toExponential(2), " ").concat(d.direction === -1 ? "(max)" : "(min)️");
            });
            // select the event detection overlay, this needs to be updated because the first useEffect
            // does not have access to the updated state of preferencePointerLocs
            var enterOverlay = selection
                .selectAll(".preferencePointerOverlay")
                .data(data);
            enterOverlay.on("click", function (event, d) {
                var match_index = data.findIndex(function (datum) { return datum.name === d.name; });
                var prefValue = (d.direction === -1
                    ? xs()[match_index]
                    : xs_rev()[match_index]).invert(pointer(event)[0]);
                // SUPER IMPORTANT TO **NOT** CHANGE STATE, BUT TO CREATE A NEW OBJECT!
                var newLocs = prefPointerLocs.map(function (loc) { return loc; });
                newLocs[match_index] = prefValue;
                setPrefPointerLocs(newLocs.map(function (loc) { return loc; }));
                setReferencePoint(newLocs.map(function (loc) { return loc; }));
            });
            enterOverlay.on("mousemove", function (event, d) {
                var match_index = data.findIndex(function (datum) { return datum.name === d.name; });
                var prefValue = (d.direction === -1
                    ? xs()[match_index]
                    : xs_rev()[match_index]).invert(pointer(event)[0]);
                // SUPER IMPORTANT TO **NOT** CHANGE STATE, BUT TO CREATE A NEW OBJECT!
                var newLocs = infoPointerLocs.map(function (loc) { return loc; });
                newLocs[match_index] = prefValue;
                setInfoPointerLocs(newLocs.map(function (loc) { return loc; }));
            });
            enterOverlay.on("mouseleave", function () {
                setInfoPointerLocs(prefPointerLocs);
            });
        }
    }, [
        selection,
        prefPointerLocs,
        infoPointerLocs,
        data,
        y,
        xs,
        xs_rev,
        xAxises,
        dimensions,
        setReferencePoint,
    ]);
    return jsx("div", { ref: ref, id: "container", className: "svg-container" });
};

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var defaultDimensions$2 = {
    chartHeight: 600,
    chartWidth: 1000,
    marginLeft: 50,
    marginRight: 50,
    marginTop: 30,
    marginBottom: 30,
};
var ParallelAxes = function (_a) {
    var objectiveData = _a.objectiveData, oldAlternative = _a.oldAlternative, dimensionsMaybe = _a.dimensionsMaybe, selectedIndices = _a.selectedIndices, handleSelection = _a.handleSelection;
    var ref = useRef(null);
    var _b = useState(null), selection = _b[0], setSelection = _b[1];
    var dimensions = useState(dimensionsMaybe ? dimensionsMaybe : defaultDimensions$2)[0];
    var _c = useState(objectiveData), data = _c[0], SetData = _c[1]; // if changes, the whole graph is re-rendered
    var _d = useState(-1), hoverTarget = _d[0], SetHoverTarget = _d[1]; // keeps track of hover target
    var _e = useState(selectedIndices), activeIndices = _e[0], SetActiveIndices = _e[1];
    var oldSol = oldAlternative;
    useEffect(function () {
        SetData(objectiveData);
    }, [objectiveData]);
    useEffect(function () {
        SetActiveIndices(selectedIndices);
    }, [selectedIndices]);
    // create an array of linear scales to scale each objective
    var ys = useCallback(function () {
        return data.ideal.map(function (_, i) {
            return scaleLinear()
                .domain([
                data.ideal[i] < data.nadir[i] ? data.ideal[i] : data.nadir[i],
                data.ideal[i] > data.nadir[i] ? data.ideal[i] : data.nadir[i], // max
            ])
                .range([dimensions.chartHeight, 0]);
        });
    }, [dimensions, data]);
    var x = useCallback(function () {
        return scaleBand()
            .domain(data.names.map(function (name) { return name; }))
            .range([0, dimensions.chartWidth])
            .padding(1);
    }, [dimensions, data]);
    var yAxixes = useCallback(function () {
        return data.directions.map(function (_, i) {
            return axisLeft(ys()[i]);
        });
    }, [data, ys]);
    useEffect(function () {
        // create a discrete band to position each of the horizontal bars
        if (!selection) {
            // add svg and update selection
            var renderH = dimensions.chartHeight + dimensions.marginBottom + dimensions.marginTop;
            var renderW = dimensions.chartWidth + dimensions.marginLeft + dimensions.marginRight;
            var newSelection = select(ref.current)
                .classed("svg-container", true)
                .append("svg")
                .attr("preserveAspectRatio", "xMinYMin meet")
                .attr("viewBox", "0 0 ".concat(renderW, " ").concat(renderH))
                //.attr("viewBox", `0 0 ${renderW} ${renderH}`)
                .classed("svg-content", true);
            // update selection
            setSelection(newSelection);
            return;
        }
        // clear the svg
        selection.selectAll("*").remove();
        var chart = selection
            .append("g")
            .attr("transform", "translate( 0 ".concat(dimensions.marginTop, ")"));
        // position the axises
        // y-axis
        data.names.map(function (name, i) {
            var axis = selection
                .append("g")
                .attr("transform", "translate(".concat(x().call(x, name) + x().bandwidth(), " ").concat(dimensions.marginTop, ")"))
                .call(yAxixes()[i].tickSizeOuter(0));
            axis.selectAll("text").attr("font-size", "20px");
            axis
                .append("text")
                .style("text-anchor", "middle")
                .attr("y", "-12px")
                .attr("font-size", "22px")
                .text(function () { return "".concat(data.names[i], " (").concat(data.directions[i] === 1 ? "min" : "max", ")"); })
                .style("fill", "black");
        });
        // create lines data
        var linesData = function (dataset) {
            return dataset.values.map(function (datum) {
                return datum.value.map(function (v, i) {
                    return [x().call(x, data.names[i]), ys()[i](v)];
                });
            });
        };
        var lines = linesData(data).map(function (datum) {
            return line()(datum.map(function (d) {
                return [d[0], d[1]];
            }));
        });
        // outline for the oldAlternative. TODO: decide on good color. Grey gets overrun by darker colors,
        if (oldSol !== undefined) {
            var oldSolution_1 = linesData(oldSol).map(function (datum) {
                return line()(datum.map(function (d) {
                    return [d[0], d[1]];
                }));
            });
            chart
                .selectAll(".oldSolution")
                .data(oldSol.values)
                .enter()
                .append("g")
                .attr("class", "oldSolution")
                .append("path")
                .attr("d", function (_, i) { return oldSolution_1[i]; })
                .style("stroke-width", 3 + "px")
                .style("stroke", "black")
                .style("fill", "none");
        }
        // add thin visual paths, these will be visible
        selection
            .append("g")
            .selectAll("path")
            .data(data.values)
            .enter()
            .append("path")
            .attr("class", "visualPath")
            .attr("transform", "translate(0 ".concat(dimensions.marginTop, ")"))
            .attr("d", function (_, i) { return lines[i]; })
            .attr("fill", "none")
            .attr("stroke", "#69b3a2")
            .attr("stroke-width", "5px");
        // add thick invisible paths, these will be used to trigger and improve mouse events
        selection
            .append("g")
            .selectAll("path")
            .data(
        // append the index to the data for easier access in event handlers
        data.values.map(function (d, i) {
            return __assign({ index: i }, d);
        }))
            .enter()
            .append("path")
            .attr("class", "pathDetector")
            .attr("transform", "translate(0 ".concat(dimensions.marginTop, ")"))
            .attr("d", function (_, i) { return lines[i]; })
            .attr("fill", "none")
            .attr("stroke", "none")
            .attr("stroke-width", "20px")
            .attr("pointer-events", "visibleStroke") // Otherwise the paths will be treated as closed shapes
            .on("mouseenter", function (_, datum) {
            SetHoverTarget(datum.index);
        })
            .on("mouseleave", function () {
            SetHoverTarget(-1);
        })
            .on("click", function (_, datum) {
            //const newData = selectedIndices.map((_, i) => data.values[i]);
            //newData[datum.index].selected = !newData[datum.index].selected;
            if (activeIndices.includes(datum.index)) {
                // remove the index
                var tmp_1 = activeIndices.filter(function (i) { return i !== datum.index; });
                handleSelection(tmp_1);
                return;
            }
            var tmp = activeIndices.concat(datum.index);
            handleSelection(tmp);
        });
    }, [selection, dimensions, data, activeIndices]);
    useEffect(function () {
        if (!selection) {
            return;
        }
        // highlight selected
        var highlightSelect = selection
            .selectAll(".visualPath")
            .data(data.values)
            .filter(function (_, i) {
            return activeIndices.includes(i);
        });
        // selection from filter is not empty
        if (!highlightSelect.empty()) {
            highlightSelect.attr("stroke-width", "5px").attr("stroke", "red");
        }
        // dim not selected
        var dimSelection = selection
            .selectAll(".visualPath")
            .data(data.values)
            .filter(function (_, i) {
            return !activeIndices.includes(i);
        });
        //  .filter((d, _) => {
        //   return !d.selected;
        //});
        // selection from filter is not empty
        if (!dimSelection.empty()) {
            dimSelection.attr("stroke-width", "5px").attr("stroke", "#69b3a2");
        }
    }, [activeIndices, selection]);
    useEffect(function () {
        if (!selection) {
            return;
        }
        // none selected, reset colors for not selected
        if (hoverTarget === -1) {
            var resetSelection = selection
                .selectAll(".visualPath")
                .data(data.values)
                .filter(function (_, i) {
                return !activeIndices.includes(i);
            });
            // check not empty
            if (!resetSelection.empty()) {
                resetSelection.attr("stroke-width", "5px").attr("stroke", "#69b3a2");
            }
            return;
            // end effect
        }
        // darken hover target
        var hoverSelection = selection
            .selectAll(".visualPath")
            .data(data.values)
            .filter(function (_, i) {
            return i === hoverTarget && !activeIndices.includes(i);
        });
        // check not empty
        if (!hoverSelection.empty()) {
            hoverSelection.attr("stroke-width", "5px").attr("stroke", "pink");
        }
    }, [hoverTarget, selection, activeIndices]);
    return jsx("div", { ref: ref, id: "container", className: "svg-container" });
};

// default is nadir in center and chart is radarChart with the circle wrapping the axises.
var defaultVisuals = {
    nadirAtCenter: true,
    radarOrSpider: true,
};
var defaultDimensions$1 = {
    chartHeight: 800,
    chartWidth: 800,
    marginLeft: 50,
    marginRight: 50,
    marginTop: 50,
    marginBottom: 50,
};
var RadarChart = function (_a) {
    var objectiveData = _a.objectiveData, oldAlternative = _a.oldAlternative, dimensionsMaybe = _a.dimensionsMaybe, userPrefForVisuals = _a.userPrefForVisuals, selectedIndices = _a.selectedIndices, handleSelection = _a.handleSelection;
    var ref = useRef(null);
    var _b = useState(null), selection = _b[0], setSelection = _b[1];
    var dimensions = useState(dimensionsMaybe ? dimensionsMaybe : defaultDimensions$1)[0];
    // color table. 0 index def color, 1 index highlight color, 2 index selected color
    var colors = ["#69b3a2", "pink", "red"];
    // set user preferences or use default settings.
    var nadirAtCenter, radarOrSpider;
    if (userPrefForVisuals === undefined) {
        nadirAtCenter = defaultVisuals.nadirAtCenter;
        radarOrSpider = defaultVisuals.radarOrSpider;
    }
    nadirAtCenter = userPrefForVisuals[0];
    radarOrSpider = userPrefForVisuals[1];
    // States
    var _c = useState(objectiveData), data = _c[0], SetData = _c[1]; // if changes, the whole graph is re-rendered
    var _d = useState(-1), hoverTarget = _d[0], SetHoverTarget = _d[1]; // keeps track of hover target
    var _e = useState(selectedIndices), activeIndices = _e[0], SetActiveIndices = _e[1];
    var oldSol = oldAlternative; // get the oldAlternative
    var offset = dimensions.marginTop + 10; // offset for margins
    var renderH = dimensions.chartHeight + dimensions.marginBottom + dimensions.marginTop;
    var renderW = dimensions.chartWidth + dimensions.marginLeft + dimensions.marginRight;
    var allAxis = data.names.map(function (name, _) { return name; }), //Names of each axis
    total = allAxis.length, //The number of different axes
    radius = Math.min(renderW / 2, renderH / 2) - offset, //Radius of the outermost circle
    angleSlice = (Math.PI * 2) / total; //The width in radians of each "slice"
    var angleDeg = 360 / total; // angleSlice in degrees
    useEffect(function () {
        SetActiveIndices(selectedIndices);
    }, [selectedIndices]);
    // this scales the axises and turns them accordingly to preferred center, default is nadir at center.
    var radScaleAxis = useCallback(function () {
        return data.ideal.map(function (_, i) {
            var start = 0;
            var end = radius;
            if (data.directions[i] === -1) {
                start = radius;
                end = 0;
            }
            if (nadirAtCenter === false) {
                start = radius;
                end = 0;
                if (data.directions[i] === -1) {
                    start = 0;
                    end = radius;
                }
            }
            return scaleLinear()
                .domain([
                data.ideal[i] < data.nadir[i] ? data.ideal[i] : data.nadir[i],
                data.ideal[i] > data.nadir[i] ? data.ideal[i] : data.nadir[i], // max
            ])
                .range([start, end]);
        });
    }, [data, nadirAtCenter, radius]);
    // this scales the data and turns them accordingly to preferred center, default is nadir at center.
    var radScaleData = useCallback(function () {
        return data.ideal.map(function (_, i) {
            var start = radius;
            var end = 0;
            if (data.directions[i] === -1) {
                start = 0;
                end = radius;
            }
            if (nadirAtCenter === false) {
                start = 0;
                end = radius;
                if (data.directions[i] === -1) {
                    start = radius;
                    end = 0;
                }
            }
            return scaleLinear()
                .domain([
                data.ideal[i] < data.nadir[i] ? data.ideal[i] : data.nadir[i],
                data.ideal[i] > data.nadir[i] ? data.ideal[i] : data.nadir[i], // max
            ])
                .range([start, end]);
        });
    }, [data, nadirAtCenter, radius]);
    // makes bandScales or each of the objectives. Sets the names and the ranges of the axises without scaling them yet.
    var rband = useCallback(function () {
        return scaleBand()
            .domain(data.names.map(function (name) { return name; }))
            .range([0, radius]);
    }, [data, radius]);
    // Makes the scaled axises. Calls radScale which will scale the objectives.
    var axisScales = useCallback(function () {
        return data.directions.map(function (_, i) {
            return axisLeft(radScaleAxis()[i]).ticks(4);
        });
    }, [data, radScaleAxis]);
    useEffect(function () {
        SetData(objectiveData);
    }, [objectiveData]);
    useEffect(function () {
        if (!selection) {
            var newSelection = select(ref.current)
                .classed("svg-container", true)
                .append("svg")
                .attr("preserveAspectRatio", "xMinYMin meet")
                .attr("viewBox", "".concat(-offset, " ").concat(-offset, " ").concat(renderW, " ").concat(renderH))
                .classed("svg-content", true);
            // update selection
            setSelection(newSelection);
            return;
        }
        // clear the svg
        selection.selectAll("*").remove();
        var centerX = renderW / 2 - offset;
        var centerY = renderH / 2 - offset;
        // offsets for ticks
        var offsetTickX = 15;
        var offsetTickY = 10;
        // helper to use with anything chart related that is not interactive with data.
        var chart = selection
            .append("g")
            .attr("transform", "translate(" + centerX + ", " + centerY + ")");
        data.names.map(function (name, i) {
            // rotates the labels horizontally
            var rotateLabels = function (i) {
                return -i * angleDeg;
            };
            // transform each axis to center of our g. For each axis we need to substract the bandwidth and rotate the axis by the angle in degrees.
            var axis = selection
                .append("g")
                .attr("class", "axises")
                .attr("transform", "translate(".concat(rband().call(rband, name) + centerX - i * rband().bandwidth(), ", 0 )\n          rotate( ").concat(i * angleDeg, " 0 ").concat(centerY, " )\n        "))
                .call(axisScales()[i].tickSizeOuter(0));
            // turn the tick labels
            axis
                .selectAll("text")
                .style("text-anchor", "start")
                .attr("transform", function () {
                if (i === 0) {
                    return "translate( ".concat(-offsetTickX - 5, " 0 ) ");
                }
                return "translate( ".concat(-offsetTickX, " 0 ) rotate(").concat(rotateLabels(i), " 0 0 ) ");
            });
            axis.selectAll("text").attr("font-size", "17px");
            axis
                .append("text")
                .attr("class", "labels")
                .style("text-anchor", "middle")
                .text(function () { return "".concat(data.names[i], " (").concat(data.directions[i] === 1 ? "min" : "max", ")"); })
                .style("fill", "black");
            // rotate the objective labels
            axis
                .selectAll(".labels")
                .attr("font-size", "18px")
                .attr("transform", function () {
                if (i === 0) {
                    return "translate(0 ".concat(-offsetTickY, " ) ");
                }
                return "translate( 30 ".concat(offsetTickY, " ) rotate(").concat(rotateLabels(i), " 0 0 ) ");
            });
        });
        // draw the outline circle for radarchart, do not for spider.
        if (radarOrSpider) {
            chart
                .selectAll("circle")
                .data(range(1, 2))
                .enter()
                .append("circle")
                .attr("r", function () { return radius; })
                .style("stroke", "black")
                .style("fill-opacity", 0.0);
        }
        // create linesdata from a dataset in polar coordinates for linesRadial
        var linesData = function (dataset) {
            return dataset.values.map(function (datum) {
                return datum.value.map(function (v, i) {
                    return [angleSlice * i, radScaleData()[i](v)]; // call radScaleData for data points
                });
            });
        };
        var linesRadial = linesData(data).map(function (datum) {
            return lineRadial().curve(curveLinearClosed)(datum.map(function (d) {
                return [d[0], d[1]];
            }));
        });
        // outline for the oldSolution
        if (oldSol !== undefined) {
            var oldSolution_1 = linesData(oldSol).map(function (datum) {
                return lineRadial().curve(curveLinearClosed)(datum.map(function (d) {
                    return [d[0], d[1]];
                }));
            });
            // draw the oldAlternative
            chart
                .selectAll(".oldSolution")
                .data(oldSol.values)
                .enter()
                .append("g")
                .attr("class", "oldSolution")
                .append("path")
                .attr("d", function (_, i) { return oldSolution_1[i]; })
                .style("stroke-width", 3 + "px")
                .style("stroke", "black")
                .style("fill", "none");
        }
        // Draw the solutions
        chart
            .selectAll(".radarArea")
            .data(data.values)
            .enter()
            .append("path")
            .attr("class", "radarArea")
            .attr("d", function (_, i) { return linesRadial[i]; })
            .attr("fill", colors[0])
            .attr("fill-opacity", 0)
            .on("mouseover", function () {
            //Dim all areas
            selectAll(".radarArea")
                .transition()
                .duration(200)
                .style("fill-opacity", 0.0);
            //Bring back the hovered over area
            select(this).transition().duration(200).style("fill-opacity", 0.8);
        })
            .on("mouseout", function () {
            //Bring back all areas
            selectAll(".radarArea")
                .transition()
                .duration(200)
                .style("fill-opacity", 0.0);
        });
        // add invis paths to trigger mouse events.
        selection
            .append("g")
            .selectAll("path")
            .data(data.values.map(function (d, i) {
            return __assign({ index: i }, d);
        }))
            .enter()
            .append("path")
            .attr("class", "pathDetector")
            .attr("transform", "translate(".concat(centerX, " ").concat(centerY, ")"))
            .attr("d", function (_, i) { return linesRadial[i]; })
            .attr("fill", "none")
            .attr("stroke", "none")
            .attr("stroke-width", "15px")
            .attr("pointer-events", "visibleStroke")
            .on("mouseenter", function (_, datum) {
            SetHoverTarget(datum.index);
        })
            .on("mouseleave", function () {
            SetHoverTarget(-1);
        })
            .on("click", function (_, datum) {
            if (activeIndices.includes(datum.index)) {
                // rm the index
                var tmp_1 = activeIndices.filter(function (i) { return i !== datum.index; });
                handleSelection(tmp_1);
                return;
            }
            var tmp = activeIndices.concat(datum.index);
            handleSelection(tmp);
        });
    }, [selection, data, dimensions, activeIndices]);
    useEffect(function () {
        if (!selection) {
            return;
        }
        // none selected, reset colors for not selected
        if (hoverTarget === -1) {
            var resetSelection = selection
                .selectAll(".radarArea")
                .data(data.values)
                .filter(function (_, i) {
                return !activeIndices.includes(i);
            });
            // check not empty
            if (!resetSelection.empty()) {
                resetSelection.attr("stroke-width", "5px").attr("stroke", "#69b3a2");
            }
            return;
            // end effect
        }
        // darken hover target
        var hoverSelection = selection
            .selectAll(".radarArea")
            .data(data.values)
            .filter(function (_, i) {
            return i === hoverTarget && !activeIndices.includes(i);
        });
        // check not empty
        if (!hoverSelection.empty()) {
            hoverSelection.attr("stroke-width", "5px").attr("stroke", "pink");
        }
    }, [hoverTarget, selection, activeIndices]);
    useEffect(function () {
        if (!selection) {
            return;
        }
        // highlight selected
        var highlightSelect = selection
            .selectAll(".radarArea")
            .data(data.values)
            .filter(function (_, i) {
            return activeIndices.includes(i);
        });
        // selection from filter is not empty
        if (!highlightSelect.empty()) {
            highlightSelect.attr("stroke-width", "5px").attr("stroke", "red");
        }
        // dim not selected
        var dimSelection = selection
            .selectAll(".radarArea")
            .data(data.values)
            .filter(function (_, i) {
            return !activeIndices.includes(i);
        });
        // selection from filter is not empty
        if (!dimSelection.empty()) {
            dimSelection.attr("stroke-width", "5px").attr("stroke", "#69b3a2");
        }
    }, [activeIndices, selection]);
    return jsx("div", { ref: ref, id: "container", className: "svg-container" });
};

var defaultDimensions = {
    chartHeight: 1200,
    chartWidth: 1200,
    marginLeft: 80,
    marginRight: 10,
    marginTop: 40,
    marginBottom: 0,
};
var NavigationBars = function (_a) {
    var problemInfo = _a.problemInfo, problemData = _a.problemData, referencePoints = _a.referencePoints, boundaries = _a.boundaries, handleReferencePoint = _a.handleReferencePoint, handleBound = _a.handleBound, handleGoBack = _a.handleGoBack, dimensionsMaybe = _a.dimensionsMaybe;
    var ref = useRef(null);
    var _b = useState(null), selection = _b[0], setSelection = _b[1];
    var dimensions = useState(dimensionsMaybe ? dimensionsMaybe : defaultDimensions)[0];
    // constants
    var data = problemData;
    var allSteps = data.totalSteps;
    var step = data.stepsTaken;
    var ideal = problemInfo.ideal;
    var nadir = problemInfo.nadir;
    var minOrMax = problemInfo.minimize;
    var objNames = problemInfo.objectiveNames;
    var offset = (dimensions.chartHeight - dimensions.marginTop - dimensions.marginBottom) /
        problemInfo.nObjectives;
    var plotHeight = offset - dimensions.marginTop; // size of individual plots
    var drawableSteps = Array.from(range(0, allSteps + 1)); // how many steps will be drawn
    var uBound = data.upperBounds;
    var ulen = uBound[0].length - 1;
    var lBound = data.lowerBounds;
    var llen = lBound[0].length - 1;
    /*===================
             Scales
        ===================*/
    // Scales the objective values to plot coordinates.
    var yAxis = useCallback(function () {
        return objNames.map(function (_, i) {
            return scaleLinear().domain([ideal[i], nadir[i]]).range([0, plotHeight]);
        });
    }, [dimensions]);
    // for returning the svg's coordinate value to parent in original scale.
    var scaleY = useCallback(function () {
        return minOrMax.map(function (_, i) {
            return scaleLinear().domain([plotHeight, 0]).range([nadir[i], ideal[i]]);
        });
    }, [dimensions]);
    var scaleX = useCallback(function () {
        return minOrMax.map(function () {
            return scaleLinear()
                .domain([0, dimensions.chartWidth])
                .range([1, allSteps]);
        });
    }, [dimensions]);
    // get the correct yAxis depending on miniming or maximizing. Reversed when maximizing. Maximal is always at the top.
    var yAxises = useCallback(function () {
        return minOrMax.map(function (_, i) {
            return axisLeft(yAxis()[i]);
        });
    }, [yAxis]);
    // xAxis handles the steps
    var xAxis = useCallback(function () {
        return objNames.map(function () {
            return scaleLinear()
                .domain([0, allSteps])
                .range([
                0,
                dimensions.chartWidth -
                    dimensions.marginRight -
                    dimensions.marginLeft,
            ]);
        });
    }, [dimensions]);
    // calls xAxis for each objective
    var xAxises = useCallback(function () {
        return objNames.map(function (_, i) {
            return axisBottom(xAxis()[i]).tickSize(0).tickValues([]);
        });
    }, [xAxis]);
    // generates the lines for refpoints and boundaries
    var lineGenerator = line()
        .x(function (d) { return d["x"]; })
        .y(function (d) { return d["y"]; })
        .curve(curveStepAfter);
    // fills pointData objects to use drawing both reference and boundary lines
    var fillPointData = function (data, drawableSteps, index) {
        var pointData = [];
        // if data is NaN we return nothing e.g. if bounds is not set it has Number.NaN in first index and we skip that objective
        if (!Number.isNaN(data[index][0])) {
            for (var _i = 0, drawableSteps_1 = drawableSteps; _i < drawableSteps_1.length; _i++) {
                var ind = drawableSteps_1[_i];
                var currentPoint = data[index][ind];
                if (currentPoint === undefined) {
                    currentPoint = data[index][step];
                }
                pointData.push({
                    x: xAxis()[index](drawableSteps[ind]),
                    y: yAxis()[index](currentPoint),
                });
            }
        }
        return pointData;
    };
    /*===================
         Main use effect.
         ===================*/
    useEffect(function () {
        if (!selection) {
            // add svg and update selection
            var renderH = dimensions.chartHeight;
            var renderW = dimensions.chartWidth;
            var newSelection = select(ref.current)
                .classed("svg-container", true)
                .append("svg")
                .attr("preserveAspectRatio", "xMinYMin meet")
                .attr("viewBox", "0 0 ".concat(renderW, " ").concat(renderH))
                .classed("svg-content", true);
            // update selection
            setSelection(newSelection);
        }
        else {
            // clear the svg of its children
            console.log("svg clear!");
            selection.selectAll("*").remove();
            // chart where to add stuff
            var chart_1 = selection
                .append("g")
                .attr("class", "chart")
                .attr("transform", "translate( ".concat(dimensions.marginLeft, " ").concat(dimensions.marginTop, ")"));
            objNames.map(function (_, i) {
                //
                chart_1
                    .append("g")
                    .attr("transform", "translate( ".concat(0, "  ").concat(offset * i, ")"))
                    .call(yAxises()[i]);
                // x
                chart_1
                    .append("g")
                    .attr("transform", "translate(0 ".concat(plotHeight + offset * i, " )"))
                    .call(xAxises()[i]);
            });
            // labels. Add them if needed.
            /*
                  chart
                    .append("g")
                    .selectAll("text")
                    .data(objNames)
                    .enter()
                    .append("text")
                    .text((d, i) => `${d} ${minOrMax[i] === -1 ? "(max)" : "(min)️"}`)
                    .attr("transform", (_, i) => {
                      return `translate( ${-70}  ${offset * i - 10})`;
                    })
                    .attr("font-size", "15px")
                    .attr("font-weight", "bold");
                    */
            // draws the polygons from upper and lowerBounds.
            var drawPolygons_1 = function (index) {
                // if minimizing else maximizing
                var j = index; // current objective index
                var i; // iterator for the steps
                var path; // path to be formed
                // first step. Needs to take first of upperBound and lowerBound
                for (i = 0; i < 1; i++) {
                    path = [
                        xAxis()[j](i),
                        yAxis()[j](uBound[j][i]),
                        xAxis()[j](i),
                        yAxis()[j](lBound[j][i]),
                    ].join(",");
                }
                // Next, we iterate through the lowerBounds
                for (i = 1; i < lBound[j].length; i++) {
                    path =
                        path + "," + [xAxis()[j](i), yAxis()[j](lBound[j][i])].join(",");
                }
                // Then, through the upperBounds
                for (i = uBound[j].length - 1; i > 0; i--) {
                    path =
                        path + "," + [xAxis()[j](i), yAxis()[j](uBound[j][i])].join(",");
                }
                return path;
            };
            var colors_1 = [
                "lightblue",
                "lightgreen",
                "lightgrey",
                "lightyellow",
                "cyan",
                "#ffb6c1",
                "#b5651d",
            ];
            // draw the polygons to the svg
            objNames.map(function (_, index) {
                var enter = selection
                    .append("g")
                    .attr("transform", "translate( ".concat(dimensions.marginLeft, " ").concat(dimensions.marginTop, ")"))
                    .selectAll("polygon")
                    .data([drawableSteps])
                    .enter();
                enter
                    .append("polygon")
                    .attr("id", "polygon")
                    .attr("transform", "translate(0 ".concat(offset * index, " )"))
                    .attr("fill", colors_1[index])
                    .attr("points", function (d) {
                    return d.map(function () { return drawPolygons_1(index); }).join(" ");
                })
                    .on("dblclick", function (e) {
                    // problem now is that e.x is not in plot coordinates..
                    var t = pointer(e)[0];
                    console.log("pointer coord", t);
                    var step = Math.floor(scaleX()[index](t));
                    console.log(step);
                    if (step < 1 || step > 100) {
                        // do nothing
                        console.log("error with step location");
                    }
                    else {
                        handleGoBack(step);
                    }
                });
                var uppLabels = selection
                    .append("g")
                    .attr("transform", "translate( ".concat(dimensions.marginLeft, " ").concat(dimensions.marginTop, ")"));
                // upper labels
                uppLabels
                    .selectAll("text")
                    .data([uBound[index][ulen]])
                    .enter()
                    .append("text")
                    .text(function (d) {
                    return "".concat(d);
                })
                    .attr("transform", function (d) {
                    return "translate( ".concat(xAxis()[index](ulen) - 30, " ").concat(yAxis()[index](d) + offset * index - 5, " )");
                })
                    .attr("font-size", "12px");
                var lowLabels = selection
                    .append("g")
                    .attr("transform", "translate( ".concat(dimensions.marginLeft, " ").concat(dimensions.marginTop, ")"));
                // lower labels
                lowLabels
                    .selectAll("text")
                    .data([lBound[index][llen]])
                    .enter()
                    .append("text")
                    .text(function (d) {
                    return "".concat(d);
                })
                    .attr("transform", function (d) {
                    if (step < 4) {
                        return "translate( ".concat(xAxis()[index](llen) - 30, " ").concat(yAxis()[index](d) + offset * index + 20, " )");
                    }
                    else {
                        return "translate( ".concat(xAxis()[index](llen) - 30, " ").concat(yAxis()[index](d) + offset * index + 10, " )");
                    }
                })
                    .attr("font-size", "12px");
            });
        }
    }, [selection, dimensions, uBound, lBound]);
    /*===================
        useEffect for boundary
        ===================*/
    useEffect(function () {
        if (!selection || !boundaries) {
            return;
        }
        selection.selectAll(".boundary").remove(); // removes old points
        selection.selectAll(".movableBound").remove(); // removes old points
        boundaries.map(function (_, index) {
            var enter = selection
                .append("g")
                .attr("transform", "translate( ".concat(dimensions.marginLeft, " ").concat(dimensions.marginTop, ")"));
            var boundaryPointData = fillPointData(boundaries, drawableSteps, index);
            var deleteOldBoundPath = function () {
                enter.selectAll(".boundary").remove();
                // remove from component's referencePointData. only for visuals
                boundaryPointData.splice(step + 1, allSteps - 1); // step + 1, step - 1
                enter
                    .selectAll(".boundary")
                    .data(boundaryPointData)
                    .enter()
                    .append("path")
                    .attr("class", "boundary")
                    .attr("d", function () { return lineGenerator(boundaryPointData); })
                    .attr("transform", "translate(0 ".concat(offset * index, " )"))
                    .attr("fill", "none")
                    .attr("stroke", "red")
                    .attr("stroke-width", "4px");
            };
            // movableLineData object
            var movableBoundData = [
                { x: xAxis()[index](step), y: 0 },
                { x: dimensions.chartWidth, y: 0 }, //. y will change so it doenst matter here
            ];
            var movePath = function () {
                // remove old movableLine
                enter.selectAll(".movableBound").remove();
                // add new movableLine and draw it
                enter
                    .selectAll(".movableBound")
                    .data(movableBoundData)
                    .enter()
                    .append("path")
                    .attr("class", "movableBound")
                    .attr("d", function () { return lineGenerator(movableBoundData); })
                    .attr("transform", "translate(0 ".concat(offset * index, " )"))
                    .attr("fill", "none")
                    .attr("stroke", "red")
                    .attr("stroke-width", "4px");
            };
            // add the boundaryLines. Also implements the drag events.
            enter
                .selectAll(".boundary")
                .data(boundaryPointData)
                .enter()
                .append("path")
                .attr("class", "boundary")
                .attr("d", function () { return lineGenerator(boundaryPointData); })
                .attr("transform", "translate(0 ".concat(offset * index, " )"))
                .attr("fill", "none")
                .attr("stroke", "red")
                .attr("stroke-width", "4px")
                .call(drag()
                .on("start", function () {
                // do nothing, if call delete here, then when clicking the line we remove the line and we dont have anything to drag.
            })
                .on("drag", function (event) {
                deleteOldBoundPath(); // delete old lines
                // get data and move while dragging
                movableBoundData[0].y = event.y;
                movableBoundData[1].y = event.y;
                movePath();
            })
                .on("end", function (event) {
                // add line coords to reference data
                var newYvalue = scaleY()[index](event.y);
                if (newYvalue > ideal[index]) {
                    newYvalue = ideal[index];
                }
                if (newYvalue < nadir[index]) {
                    newYvalue = nadir[index];
                }
                // SUPER IMPORTANT TO **NOT** CHANGE STATE, BUT TO CREATE A NEW OBJECT!
                var newBounds = boundaries.map(function (bound) { return bound; });
                newBounds[index][step] = newYvalue;
                handleBound(newBounds); // call the refence handler
            }));
        });
    }, [selection, boundaries, handleBound]);
    /*===================
        useEffect for refLines
        ===================*/
    useEffect(function () {
        if (!selection) {
            return;
        }
        selection.selectAll(".refPoint").remove(); // removes old points
        selection.selectAll(".movableLine").remove(); // removes old points
        referencePoints.map(function (_, index) {
            var enter = selection
                .append("g")
                .attr("transform", "translate( ".concat(dimensions.marginLeft, " ").concat(dimensions.marginTop, ")"));
            var referencePointData = fillPointData(referencePoints, drawableSteps, index);
            var deleteOldLinePath = function () {
                enter.selectAll(".refPoint").remove();
                // remove from component's referencePointData. only for visuals
                referencePointData.splice(step + 1, allSteps - 1); // step + 1, step - 1
                enter
                    .selectAll(".refPoint")
                    .data(referencePointData)
                    .enter()
                    .append("path")
                    .attr("class", "refPoint")
                    .attr("d", function () { return lineGenerator(referencePointData); })
                    .attr("transform", "translate(0 ".concat(offset * index, " )"))
                    .attr("fill", "none")
                    .attr("stroke", "darkgreen")
                    //.attr("stroke-dasharray", "4,2")
                    .attr("stroke-width", "5px");
            };
            // movableLineData object
            var movableLineData = [
                { x: xAxis()[index](step), y: 0 },
                { x: dimensions.chartWidth, y: 0 }, //. y will change so it doenst matter here
            ];
            var movePath = function () {
                // remove old movableLine
                enter.selectAll(".movableLine").remove();
                // add new movableLine and draw it
                enter
                    .selectAll(".movableLine")
                    .data(movableLineData)
                    .enter()
                    .append("path")
                    .attr("class", "movableLine")
                    .attr("d", function () { return lineGenerator(movableLineData); })
                    .attr("transform", "translate(0 ".concat(offset * index, " )"))
                    .attr("fill", "none")
                    .attr("stroke", "darkgreen")
                    //.attr("stroke-dasharray", "4,2")
                    .attr("stroke-width", "5px");
            };
            // add the referenceLines. Also implements the drag events.
            enter
                .selectAll(".refPoint")
                .data(referencePointData)
                .enter()
                .append("path")
                .attr("class", "refPoint")
                .attr("d", function () { return lineGenerator(referencePointData); })
                .attr("transform", "translate(0 ".concat(offset * index, " )"))
                .attr("fill", "none")
                .attr("stroke", "darkgreen")
                //.attr("stroke-dasharray", "4,2")
                .attr("stroke-width", "5px")
                .call(drag()
                .on("start", function () {
                // do nothing, if call delete here, then when clicking the line we remove the line and we dont have anything to drag.
            })
                .on("drag", function (event) {
                deleteOldLinePath(); // delete old lines
                // get data and move while dragging
                movableLineData[0].y = event.y;
                movableLineData[1].y = event.y;
                movePath();
            })
                .on("end", function (event) {
                // add line coords to reference data
                var newYvalue = scaleY()[index](event.y);
                if (minOrMax[index] === -1) {
                    if (newYvalue > ideal[index]) {
                        newYvalue = ideal[index];
                    }
                    if (newYvalue < nadir[index]) {
                        newYvalue = nadir[index];
                    }
                }
                else {
                    //minimizing
                    if (newYvalue < ideal[index]) {
                        newYvalue = ideal[index];
                    }
                    if (newYvalue > nadir[index]) {
                        newYvalue = nadir[index];
                    }
                }
                // SUPER IMPORTANT TO **NOT** CHANGE STATE, BUT TO CREATE A NEW OBJECT!
                var newRefPoints = referencePoints.map(function (ref) { return ref; });
                newRefPoints[index][step] = newYvalue;
                handleReferencePoint(newRefPoints); // call the refence handler
            }));
        });
    }, [selection, handleReferencePoint, referencePoints]);
    return jsx("div", { ref: ref, id: "container", className: "svg-container" });
};

export { HorizontalBars, NavigationBars, ParallelAxes, RadarChart };
